package com.blazinggames.ootw.ep31
{
	// Adobe Imports
	import flash.display.MovieClip;

	public class MapRenderer
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

// *********************
// ***** VARIABLES *****
// *********************

		public var _displayWidth:int;
		public var _displayHeight:int;
		protected var _slicesInCircle:int;
		protected var _sliceSize:Number;
		
		protected var _castTileX:Array;
		protected var _castTileY:Array;
		protected var _castDistance:Array;
		protected var _castHeightTable:Array
		protected var _castHitX:Array;
		protected var _castHitY:Array;
		
		// for map management
		private var _map:Array;
		private var _mapsize:int;
		
// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function MapRenderer()
		{
			_slicesInCircle = 3200;//2560;
			_sliceSize = 2.0 * Math.PI / Number(_slicesInCircle);
			_displayWidth = 640;
			_castTileX = new Array(_displayWidth);
			_castTileY = new Array(_displayWidth);
			_castHitX = new Array(_displayWidth);
			_castHitY = new Array(_displayWidth);
			_castDistance = new Array(_displayWidth);
			_castHeightTable = new Array(_displayWidth);
			
			// build height distortion compensation table
			var angle:Number = -_displayWidth / 2 * _sliceSize;
			for (var cntr:int = 0; cntr < _displayWidth; ++cntr) {
				_castHeightTable[cntr] = 400 / Math.cos(angle);
				angle += _sliceSize;
			}
/*			
			// build the map
			_map = new Array(1024);
			for (cntr = 0; cntr < 1024; ++cntr)
				_map[cntr] = TILES.indexOf(RAW_MAP.charAt(cntr));
*/
		}
		
// ***** Map Management - should be a separate class but this is just a test

		public function setMap(map:Array, mapsize:int) 
		{
			
			_map = map;
			_mapsize = mapsize;
		}
		
// **********************************
// ***** Functions - Raycasting *****
// **********************************

		protected function performCast(
				playerX:Number,
				playerY:Number,
				playerD:Number)
		{
			var degreesToSlices:Number = playerD * _slicesInCircle / 360
			var angle:Number = (degreesToSlices - _displayWidth / 2) * _sliceSize;
//			var angle:Number = (playerD - 45) * _sliceSize; // todo - make proper
			var startTileX:int = int(playerX);
			var startTileY:int = int(playerY);
			var tileX:int, tileY:int, tileIndex;
			var tileStepX:int, tileStepY:int;
			var stepX:Number, stepY:Number;
			var stepSizeX:Number, stepSizeY:Number;
			var distanceX:Number, distanceY:Number, distance:Number;
			
			for (var cntr:int = 0; cntr < _displayWidth; ++cntr)
			{
				tileX = startTileX;
				tileY = startTileY;
				tileIndex = startTileY * _mapsize + startTileX;
				
				// first, find the size of each step and the distance to the first step
				stepX = Math.sin(angle);
				if (stepX == 0) {
					stepSizeX = 10000000; // big number
					tileStepX = 1; // shouldn't matter
					distanceX = stepSizeX;
				} else if (stepX < 0) {
					stepSizeX = -1/stepX;
					tileStepX = -1;
					distanceX = (playerX - Number(startTileX)) * stepSizeX;
				} else {
					stepSizeX = 1/stepX;
					tileStepX = 1;
					distanceX = (Number(startTileX+1) - playerX) * stepSizeX;
				}
				stepY = -Math.cos(angle);
				if (stepY == 0) {
					stepSizeY = 10000000; // big number
					tileStepY = -1;
					distanceY = stepSizeY;
				} else if (stepY < 0) {
					stepSizeY = -1/stepY;
					tileStepY = -1;
					distanceY = (playerY - Number(startTileY)) * stepSizeY;
				} else {
					stepSizeY = 1/stepY;
					tileStepY = 1; // y is the big step
					distanceY = (Number(startTileY+1) - playerY) * stepSizeY;
				}
				
				// now keep stepping through tiles until we hit a wall
				do {
					if (distanceX < distanceY) {
						tileX += tileStepX;
						tileIndex += tileStepX;
						distance = distanceX;
						distanceX += stepSizeX;
					} else {
						tileY += tileStepY;
						tileIndex += (tileStepY * _mapsize);
						distance = distanceY;
						distanceY += stepSizeY;
					}
				} while (_map[tileIndex] < 4);
				_castTileX[cntr] = tileX;
				_castTileY[cntr] = tileY;
				_castDistance[cntr] = distance;
				
				_castHitX[cntr] = playerX + distance * stepX;
				_castHitY[cntr] = playerY + distance * stepY;
				
				// make sure we have next angle
				angle += _sliceSize;
			}
		}

// ******************************************
// ***** Functions - Display Generation *****
// ******************************************

		/**
		* While this is intended as a test function, it could be useful as an
		* overlay in the automap. Simply put, this does the ray projection but
		* instead of generating a 3D view, it instead generates an alpha 
		* overlay showing on the map where the rays are projecting to.
		*/
		public function renderVisibility(
				playerX:Number,
				playerY:Number,
				playerD:Number,
				tileSize:int = 32)
		{
			var startTileX:int = int(playerX);
			var startTileY:int = int(playerY);
			var startAngle = (playerD - 45) * _sliceSize;
			var endAngle = (playerD + 45) * _sliceSize;
			performCast(playerX, playerY,playerD);
			graphics.clear();
			graphics.lineStyle(3, 0xFF00FF, 0.5);
			for (var cntr:int = 0; cntr < _displayWidth; ++cntr){
				graphics.moveTo(playerX * tileSize, playerY * tileSize);
//				graphics.lineTo(_castTileX[cntr] * tileSize, _castTileY[cntr] * tileSize);
				graphics.lineTo(_castHitX[cntr] * tileSize, _castHitY[cntr] * tileSize);				
			}

			graphics.lineStyle(3, 0x0000FF, 0.5);
			graphics.moveTo(playerX * tileSize, playerY * tileSize);
			graphics.lineTo(Math.sin(startAngle)*100+playerX*tileSize,
						-Math.cos(startAngle)*100+playerY*tileSize);
			graphics.moveTo(playerX * tileSize, playerY * tileSize);
			graphics.lineTo(Math.sin(endAngle)*100+playerX*tileSize,
						-Math.cos(endAngle)*100+playerY*tileSize);
		}

		public function renderSolid( 
					playerX:Number,
					playerY:Number,
					playerD:Number)
		{
			var hght:Number;
			var half:Number;
			var colour:uint;
			performCast(playerX, playerY,playerD);
			graphics.clear();
			graphics.lineStyle(1, 0xFF00FF);
			for (var cntr:int = 0; cntr < _displayWidth; ++cntr){
				hght = _castHeightTable[cntr] / (.000001 + _castDistance[cntr]);
				half = hght / 2;
				if (half > 220)
					graphics.lineStyle(1, 0xe0e0ef);
				else {
					colour = uint(half);
					graphics.lineStyle(1, (colour << 16) | (colour << 8) | (colour + 15));
				}
				graphics.moveTo(cntr,  240 - half);
//				graphics.lineTo(_castTileX[cntr] * tileSize, _castTileY[cntr] * tileSize);
				graphics.lineTo(cntr, 240 + half);				
			}
			
		}
		
	} // END CLASS	
} // END PACKAGE
