﻿package com.blazinggames.ootw.ep31
{
	// Adobe Imports
	import flash.display.Graphics;
	import flash.display.MovieClip;

	public class Maze
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public const TILES:String = "-cxano";
		public const TILE_EMPTY = 0;
		public const TILE_START = 1;
		public const TILE_END = 2;
		public const TILE_HOLE = 3;
		public const TILE_WALL = 4;
		public const TILE_ALSOWALL = 5;
		public const TILE_UNKNOWN = 6;
		public const RAW_MAP:String =
"nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn"+
"nxxxxxxx-----------------a-----n"+
"n-nnnnnn-nnnnnnnannnnnnnnn-nnn-n"+
"nnnnnnnn-nnnnnnn-nnnnnnnnn-nnn-n"+
"nnnnnnnn-nnnnnnn-nnnnnnnnn-nnn-n"+
"nnnnnnnn-------------------nnn-n"+
"nnnnnnnnnnnn-nnnnnnn-nnnnn-nnn-n"+
"nnnnnnnnnnnn-nnnnnnn-nnnnn-nnn-n"+
"n---a-------a--------------nnn-n"+
"n-nnnnnn-nnnnnn-nnnnnnnnnn-nnn-n"+
"n-nnnnnn-nnnnnn-nnnnnnnnnn-nnn-n"+
"n-nnnnnn-nnnnnnannnnnnnnnn-----n"+
"n-nnnnnn-nnnnnn-nnnnnnnnnn-nnn-n"+
"n-nnnnnn-nnnnnn-nnnnnnnnnn-nnn-n"+
"n--------oooooo-nnnnnnnnnn-nnn-n"+
"n-nnnnno-----------------------n"+
"n-nnnnno-ooo-onnnnnn-nnnnn-nnn-n"+
"n-nnnnno-ooo-onnnnnn-nnnnn-nnn-n"+
"n-oooooo-ooo-onnnnnn-nnnnn-nnn-n"+
"n-----a--------------------nnn-n"+
"nooo-ooooooo-on-nnnnnnnnnn-nnnan"+
"nooo-ooooooo-on-nnnnnnnnnn-nnn-n"+
"nooo-ooooooo-on-nnnnnnnnnn-nnn-n"+
"nooo-ooooooo-oo-oooooooooo-ooo-n"+
"n-------------------------a----n"+
"n-ooooo-ooooooo-oon-nno-oo-ooo-n"+
"n-ooooo-ooooooo-oon-nno-oo-ooo-n"+
"n-ooooo-ooooooo-oon-nno--------n"+
"n-ooooo-ooooooo-oon-nnn-ooooaoon"+
"n------a--------oon-nnn-oooo-oon"+
"nooooooooooooooooon-----------cn"+
"nnnnnnnnnnnnnnnnnnnnnnnnnnnnnnnn";

		protected const AUTOMAPCOLORS:Array = new Array(
			0xffffff, 0xffff00, 0x00ffff, 0xff0000,0,0,0x808080
		);
		public const PLAYERSTARTX:Number = 30.5;
		public const PLAYERSTARTY:Number = 30.5;
		public const PLAYERSTARTD:Number = 270;

		public const STATE_LOSE:int = -1;
		public const STATE_NORMAL:int = 0;
		public const STATE_WIN:int = 1;
		
		public const DEGREEINRADS:Number = Math.PI / 180;

// *********************
// ***** VARIABLES *****
// *********************

		// for map management
		private var _map:Array;
		private var _automap:Array;
		
		// for player management
		var _player_x:Number;
		var _player_y:Number;
		var _player_dir:Number;

		// _render_movie is part of movie clip
		// _automap_movie is part of movie clip

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function Maze()
		{
			var cntr:int;
			
			// build the map
			_map = new Array(1024);
			_automap = new Array(1024);
			for (cntr = 0; cntr < 1024; ++cntr)
				_map[cntr] = TILES.indexOf(RAW_MAP.charAt(cntr));
			_render_movie.setMap(_map, 32);
			resetMap(true);
		}

// **************************************
// ***** Functions - Map Management *****
// **************************************

		public function getTile(x:uint, y:uint):int
		{
			var mapX = x & 31;
			var mapY = y & 31;
			return _map[(mapY*32) + mapX];
		}

		public function resetMap(clearAuto:Boolean = false)
		{
			_player_x = PLAYERSTARTX;
			_player_y = PLAYERSTARTY;
			_player_dir = PLAYERSTARTD;
			_render_movie.renderSolid(_player_x, _player_y, _player_dir);
			if (clearAuto) {
				for (var cntr:int = 0; cntr < 1024; ++cntr)
					_automap[cntr] = TILE_UNKNOWN;
			}
			updateAutomap();
		}

		public function generateAutomap()
		{
			var tileIndex:int = 0;
			var g:Graphics = _automap_movie.graphics;
			g.clear();
			for (var cntrY:int = 0; cntrY < 32; ++cntrY) {
				for (var cntrX:int = 0; cntrX < 32; ++cntrX) {
					g.beginFill(AUTOMAPCOLORS[_automap[tileIndex]]);
					g.drawRect(cntrX*4, cntrY*4, 4, 4);
					g.endFill();
					++tileIndex;
				}
			}
			g.beginFill(0xff00ff);
			g.drawRect(_player_x * 4 - 2, _player_y * 4 -2, 5, 5);
			g.endFill();
		}
		
		/**
		* This is tricky as we want to keep traps secret until they are sprung
		* but keep the traps visible if the player keeps the map. This is why
		* we force the player's location tile onto the automap but only change
		* non-set tiles and even then treat the trap tile as a space.
		*/
		public function updateAutomap()
		{
			var tileIndex:int;
			var tx:int, ty:int;
			for (var cntrX:int = -1; cntrX < 2;++cntrX) {
				tx = int(_player_x) + cntrX;
				for (var cntrY:int = -1; cntrY < 2; ++cntrY) {
					ty = int(_player_y) + cntrY;
					tileIndex = ty * 32 + tx;
					if (_automap[tileIndex] == TILE_UNKNOWN)
						if (_map[tileIndex] == TILE_HOLE)
							_automap[tileIndex] = TILE_EMPTY;
						else
							_automap[tileIndex] = _map[tileIndex];
				}
			}
			tileIndex = int(_player_y) * 32 + int(_player_x);
			_automap[tileIndex] = _map[tileIndex];
			generateAutomap();
		}

// *****************************************
// ***** Functions - Player Management *****
// *****************************************

		public function moveForward(dist:Number):int
		{
			var angle:Number = _player_dir * Math.PI/180;
			var nx:Number = _player_x + dist * Math.sin(angle);
			var ny:Number = _player_y - dist * Math.cos(angle);
			var nt:int = getTile(uint(nx), uint(ny));
			if (nt < TILE_WALL) {
				var chck:Boolean = 
						( (Math.floor(nx) != Math.floor(_player_x)) ||
						(Math.floor(ny) != Math.floor(_player_y)));
				_player_x = nx;
				_player_y = ny;
				if (chck)
					updateAutomap();
			}
			_render_movie.renderSolid(_player_x, _player_y, _player_dir);
			if (nt == TILE_END)
				return STATE_WIN;
			else if (nt == TILE_HOLE)
				return STATE_LOSE;
			else
				return STATE_NORMAL
		}
		
		public function turnLeft(dist:Number)
		{
			_player_dir -= (dist*90);
			_render_movie.renderSolid(_player_x, _player_y, _player_dir);
		}
		
		public function turnRight(dist:Number)
		{
			_player_dir += (dist*90);
			_render_movie.renderSolid(_player_x, _player_y, _player_dir);
		}
	} // END CLASS
} // END PACKAGE
